#!/usr/bin/perl
=head1 NAME

  postprocess.pl - Post processes the coseg results 

=head1 SYNOPSIS

  postprocess.pl -s <sequences file> -i <insertions file> 
                 -c <consensus file>

=cut

use strict;
use Getopt::Long;
use Data::Dumper;


## TODO: Create cytoscape *.sif file:
##              Node#<tab>-<tab>Node#
##              ..
##       Create cytoscape *.noa files ( one for each parameter ):
##              pVal (java.lang.String)
##              Node# = Value
##              ..       
## Could also create an xgmml file -- which has all these things together:
##     http://www.cs.rpi.edu/~puninj/XGMML/
##<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
##<graph label="cyto.sif.1" xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:xlin
##k="http://www.w3.org/1999/xlink" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syn
##tax-ns#" xmlns:cy="http://www.cytoscape.org" xmlns="http://www.cs.rpi.edu/XGMML"
## >
##  <att name="documentVersion" value="1.1"/>
##  <att name="networkMetadata">
##    <rdf:RDF>
##      <rdf:Description rdf:about="http://www.cytoscape.org/">
##        <dc:type>Protein-Protein Interaction</dc:type>
##        <dc:description>N/A</dc:description>
##        <dc:identifier>N/A</dc:identifier>
##        <dc:date>2008-07-24 11:08:00</dc:date>
##        <dc:title>cyto.sif.1</dc:title>
##        <dc:source>http://www.cytoscape.org/</dc:source>
##        <dc:format>Cytoscape-XGMML</dc:format>
##      </rdf:Description>
##    </rdf:RDF>
##  </att>
##  <att type="string" name="backgroundColor" value="#ffffff"/>
##  <node label="237" id="-249">
##    <att type="real" name="divergence" value="0.14"/>
##    <att type="integer" name="p" value="2"/>
##    <att type="string" name="subfamily" value="sub237"/>
##    <att type="real" name="pVal" value="0.1"/>
##    <att type="integer" name="count" value="94"/>
##    <att type="string" name="mutations" value="235:t-&amp;amp;gt;g"/>
##    <att type="string" name="canonicalName" value="237"/>
##  </node>
##  <edge label="237 (-) 236" source="-249" target="-246">
##    <att type="string" name="XGMML Edge Label" value="237 (-) 236"/>
##    <att type="string" name="interaction" value="-"/>
##    <att type="string" name="canonicalName" value="237 (-) 236"/>
##    <graphics width="1" fill="#0000ff" cy:sourceArrow="0" cy:targetArrow="0" cy:sourceArrowColor="#000000" cy:targetArrowColor="#000000" cy:edgeLabelFont="Default-0-10" cy:edgeLineType="SOLID" cy:curved="STRAIGHT_LINES"/>
##  </edge>
##</graph>


#
# Option processing
#  e.g.
#   -t: Single letter binary option
#   -t=s: String parameters
#   -t=i: Number paramters
#
my @getopt_args = (
    '-s=s', 
    '-c=s',
    '-i=s'
);

my %options = ();
Getopt::Long::config("noignorecase", "bundling_override");
unless (GetOptions(\%options, @getopt_args)) {
    usage();
}

sub usage {
  exec "pod2text $0";
  exit;
}

usage() if ( ! $options{'s'} || ! $options{'i'} || ! $options{'c'} );


# input file listing insertion values of each element
my $insertSourceFile = $options{'i'};  

# input file of assignments to all subfamilies 
my $assignFile = $options{'s'} . ".assign"; 

# Consensus variation output from program.
my $subFile = $options{'s'} . ".subfamilies";     # input file of subfamilies

# Original consensus file
my $consFile = $options{'c'};          # input file of AluSx

# Main log file
my $logFile = $options{'s'} . ".log";

# Graph file
my $vizFile = $options{'s'} . ".tree.viz";

# Temporary output file
my $tmpInsertConsFile = $options{'s'} . ".subfamilies.ins";   # resolved insertions

# Final sequence file for each subfamily with insertions resolved.
my $seqFile = $options{'s'} . ".subfamilies.seq";   


print "Postprocessing coseg results\n";

##
##  Read in mutation information
##
my @parent = ();
open MUT, "<$logFile" ||
   die "Error: Could not open file $logFile\n\n";
my @mutations = ();
while ( <MUT> )
{
  if ( /Building subfamily\s+(\d+)\s+\(parent\s+(\d+), logpvalue.*:\s+(.*)/ )
  {
    my $subfam = $1;
    my $muts = $3;
    $parent[$1] = $2;
    my $mutString = "parent=$2 muts=";
    while ( $muts =~ /pos\s+(\d+)\s+(\S)\s+to\s+(\S)/ig )
    {
      $mutString .= " $1:$2/$3";
    } 
    $mutations[$subfam] = $mutString;
  }
}
close MUT;
 
 
## 
## Read in pvalues
##
my @pvalues = ();
# First the scaffold pvalues
open SUB, "<$subFile" ||
   die "Error: Could not open file $subFile\n\n";
while ( <SUB> )
{
 if ( /Subfamily\s+(\d+):.*(mstLogPValue|logpvalue)\s+([-\d\.]+)/ ) 
 {
   my $aa = abs($3*.434294481);
   my $b = int($aa);
   my $c = $aa - $b;
   my $d = exp(-$c/.434294481);
   my $x; 
   my $y;
   if($d >= 0.95)
   {
     $x = 1;
     $y = -$b;
   }
   else
   {
     $x = int(10*$d + 0.5);
     $y = -($b+1);
   }
   $pvalues[$1] = "${x}e$y";
 } 
}
close( LISTFILE );


##
## Build the insert file
##
open TMPINSERT, ">$tmpInsertConsFile" || 
   die("Error: Could not open file for writing: $tmpInsertConsFile!\n\n");
open(SUB,"$subFile") || 
   die "Error: Could not open file $subFile\n\n";
my $s = 0;
my @mutrates = ();
my $consensus;
my $consensuscount;
my $n;
my @count = ();
while( <SUB> )
{
  #Subfamily 4: count 739 mutrate 0.027/0.023 mstLogPValue -413.400874
  #Subfamily 58: count 129 mutrate 0.206/0.200 sigma 17.839535, logpvalue -149.335610
  if ( /^Subfamily\s+(\d+):\s+count\s+\d+\s+mutrate\s+([\d\.]+)\/([\d\.]+)/ )
  { 
    print TMPINSERT $_;
    $mutrates[$1] = $2;
    next;
  }

  chomp;
  my @array = split / +/;
  for( my $l=0; $l<=$#array; $l++ )
  {
    print TMPINSERT "$array[$l]";
    # If "+" appears in the a subfamily entry
    if( $array[$l] =~ /(\d+):\+/ )
    {
      # NOW, print insertion consensus for subfamily $s position $x

      # The insertion position
      my $insPos = $1;
 
      print "  Found an insertion site: $array[$l]\n";

      # The assignment file and the insertion sequences files
      #print "Opening $insertSourceFile\n";
      open INSERTS,"<$insertSourceFile" || 
          die "Error: Could not open file $insertSourceFile\n\n";
      open ASSIGNS,"$assignFile" || 
          die "Error: Could not open file $assignFile\n\n";

      # Clear consensus variables
      $consensus = "~~~";
      $consensuscount = 0;
      $n = 0;

      print "    Calculating consensus for insertion site:";
      while(<ASSIGNS>)
      {
        print "." if ( ( $n % 1000 ) ==  0 );
        if ( /(\d+)\s+(\d+)/ ) 
        {
          my $lineb = <INSERTS>;
          if ( $1 != $n ) 
            { print("Problem with assign file $1 != $n\n"); exit(1); }
          if ( $2 != $s ) 
            { $n++; next; }

          #print $_;

          chomp $lineb;
          my @insArray = split(/ +/,$lineb);
          for(my $l4=0; $l4<=$#insArray; $l4++)
          {
            if ( $insArray[$l4] =~ /(\d+):(\S+)/ )
            {
              #print "     Trying $insArray[$l4]\n";
              my $pos = $1; 
              my $seq = $2;
              if( $pos != $insPos ) { next; }
              $count[$s][$insPos]{$seq}++;
              if($count[$s][$insPos]{$seq} > $consensuscount)
              {
                $consensuscount = $count[$s][$insPos]{$seq};
                $consensus = $seq;
              }
            }
          }
          $n++;
        }
      }
      print "\n";
      print "    Consensus = $consensus\n";
      if($consensus =~ /\~/) 
         { die "OOPS s=$s x=$insPos consensus=$consensus\n"; }
      print TMPINSERT "$consensus";
      close( INSERTS );
      close( ASSIGNS );
    }
    print TMPINSERT " ";
  }
  print TMPINSERT "\n";
  $s++;
}
close( SUB );
close( TMPINSERT );

# next, build seqFile
open TMPINSERT,"<$tmpInsertConsFile" || 
    die "Error: Could not re-open file $tmpInsertConsFile\n\n";
open ASSIGNS,"$assignFile" || 
    die "Error: Could not re-open file $assignFile\n\n";
open(SEQ,">$seqFile") || die("COF");
open CONS,"<$consFile" ||
   die "Error: Could not open file $consFile\n\n";

my @ccount = ();
while(my $line = <ASSIGNS>)
{
  my @array = split(/ +/,$line);
  $ccount[$array[1]] += 1;
}
close(ASSIGNS);
my $maxCount = 0;
foreach my $count ( @ccount )
{
  $maxCount = $count if ( $maxCount < $count );
}

my $conSeq;
while(my $line = <CONS>)
{
  if($line =~ />/) { next; }
  chomp($line);
  $conSeq .= lc($line);
}
my $L = length( $conSeq );
my @array = split(//,$conSeq);
my @Sx = ();
for(my $x=0; $x<$L; $x++) { $Sx[$x] = $array[$x]; }
my $n=0;
my @this = ();
my @this2 = ();
my @array2 = ();
while(my $line = <TMPINSERT>)
{
  for(my $x=0; $x<$L; $x++) { $this[$x] = $Sx[$x]; $this2[$x] = $Sx[$x]; }
  $line = <TMPINSERT>;
  chomp($line);
  @array = split(/ +/,$line);
  my $length = @array;
  for(my $l=0; $l<$length; $l++)
  { 
    @array2 = split(/:/,$array[$l]);
    my $x = $array2[0];
    my $string = $array2[1];
    if($string =~ /\+/)
    {
      $string =~ s/\+//;
      $this[$x] .= $string;
      $this2[$x] .= "+";
    }
    else
    {
      $this2[$x] = $string;
      $string =~ s/-//;
      $this[$x] = $string;
    }
  }

  print SEQ ">subfamily$n count=$ccount[$n] pvalue=$pvalues[$n]";
  if ( $parent[$n] >= 0 )
  {
    print SEQ " ( parent_pvalue=$pvalues[$parent[$n]] )";
  }
  print SEQ " " . $mutations[$n];
  print SEQ "\n";

  for(my $x=0; $x<$L; $x++) { print SEQ "$this[$x]"; }
  printf SEQ ("\n");

  $n++;
}
close( TMPINSERT );
close( ASSIGNS );
close( SEQ );
close( CONS );
print("Done building $seqFile\n");
unlink( $tmpInsertConsFile );



## REBUILD VIZ FILE
open VIZ, "<$vizFile" ||
   die "Could not open $vizFile\n";
open OUT ,">$vizFile.new" ||
   die "Could not open $vizFile.new\n";

# Find max/min divergence
my $maxDiv = 0;
my $minDiv = 1;
foreach my $mutrate ( @mutrates )
{
  $maxDiv = $mutrate if ( $mutrate > $maxDiv );
  $minDiv = $mutrate if ( $mutrate < $minDiv );
}
my $divRange = $maxDiv - $minDiv;
$divRange = 1.0 if ( $divRange == 0 );

while ( <VIZ> )
{
  if ( /^(\s+)(\d+)(.*label=\")\d+(\",.*)$/ )
  {
    my $color = sprintf("%0.2f", (($mutrates[$2] - $minDiv)*0.6)/$divRange);

    my $newLine =  $1 . $2 . $3 .
            "sub$2\\nc=" . $ccount[$2] . "\\npv=" . $pvalues[$2] . "\\n" .
            "div=" . $mutrates[$2] . "\\n" . $mutations[$2] .  $4 . "\n";
    my $subfam = $2;
    if ( $newLine =~ /^(.*)height=[\d\.]+,\s+width=[\d\.]+,(.*)$/ )
    {
      my $size = 2 * sqrt($ccount[ $subfam ] / $maxCount);
      $newLine = $1 . "height=$size, width=$size," . $2 . "\n";
    }
    if ( $newLine =~ /style=solid/ )
    {
      $newLine =~ s/style=solid/style=filled/;
      my $hsv = "$color,0.15,0.95";
      $newLine =~ s/color=(red|grey|green|blue)/color="$hsv"/;
    }else {
      my $hsv = "$color,0.9,0.6";
      $newLine =~ s/color=(red|grey|green|blue)/color="$hsv"/;
    }
    print OUT $newLine;
  }else
  {
    print OUT $_;
  }
}
close VIZ;
close OUT; 

if ( -s "$vizFile.new" )
{
  system ( "mv $vizFile.new $vizFile" );
}

1;
